package WebService::Async::Onfido::Photo;

use strict;
use warnings;

use parent qw(WebService::Async::Onfido::Base::Photo);

our $VERSION = '0.003';    # VERSION

=head1 NAME

WebService::Async::Onfido::Photo - represents data for Onfido

=head1 DESCRIPTION

=cut

1;

__END__

=head1 AUTHOR

deriv.com C<< DERIV@cpan.org >>

=head1 LICENSE

Copyright deriv.com 2019. Licensed under the same terms as Perl itself.

